/*
** Module   : Container.java
** Abstract : Common interface for widget container classes.
**
** Copyright (c) 2010, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100000          Created initial version
** 002 SIY 20100804          Changes in imports, some refactorings.
** 003 SIY 20100831          Changes related to refactoring of FrameHelper.
*/

package com.goldencode.p2j.ui.client.widget;

import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.ui.client.layout.*;

/**
 * Common interface implemented by all widget containers.
 */
public interface Container
extends Widget
{
   /**
    * Add widget to container.
    * 
    * @param   widget
    *          Widget to add.
    *          
    * @return  <code>this</code> for fluent syntax support.
    */
   public Container add(Widget widget);

   /**
    * This method should be overridden in a container that allows clipping.
    * 
    * @return  <code>true</code> if container clips its children. 
    */
   public boolean clipping();

   /**
    * Get widget which is in focus.
    * 
    * @return  focused widget.
    */
   public Widget currentFocus();

   /**
    * Get default focusable widget.
    * 
    * @return  default focusable widget.
    */
   public Widget defaultFocus();

   /**
    * Lay out container.
    */
   public void doLayout();

   /**
    * Indicates the ending of the resize process. All repainting between
    * <code>startSizeChange</code> and <code>endSizeChange</code> calls is
    * suppressed.
    * 
    * @param    repaint
    *           If <code>true</code> then the current bounding rectangle and
    *           the rectangle saved during <code>startSizeChange</code> call
    *           are repainted.
    */
   public void endSizeChange(boolean repaint);

   /**
    * Returns the focused direct child Component.
    * 
    * @return   The focused direct child Component.
    */
   public Widget focus();

   /**
    * Get widget for provided index or <code>null</code> if no such component
    * exists.
    * 
    * @param   index
    *          Widget index.
    *           
    * @return  widget at specified index or <code>null</code> if index is out
    *          of range.
    */
   public Widget get(int index);

   /**
    * Get clipping rectangle.
    * 
    * @return   clipping rectangle.
    */
   public Rectangle getClipRect();

   /**
    * Reports if there is at least 1 focusable widget that is not the given
    * widget.
    * 
    * @param    exclude
    *           A widget to ignore or <code>null</code> if any focusable
    *           widget will do.
    * 
    * @return   <code>true</code> if there is at least one other widget that
    *           can be focused.
    */
   public boolean hasFocusable(Widget exclude);

   /**
    * Get container insets.
    *  
    * @return  container insets.
    */
   public Insets insets();
   
   /*
    * TODO this can be a client side implementation
    * 
    * Mode specified widget before the widget "before".
    * 
    * @param   widget
    *          Widget to mover.
    * @param   before
    *          The widget before to insert mover.
    *
   public void moveBefore(Widget widget, Widget before);
   */

   /**
    * Mode specified widget to the bottom of the widget list.
    * 
    * @param   widget
    *          Widget to mover.
    */
   public void moveToBottom(Widget widget);

   /**
    * Move specified widget to the top of widget list.
    * 
    * @param   widget
    *          Widget to mover.
    */
   public void moveToTop(Widget widget);

   /**
    * Move widget to specified position in the widget list. For overlaying 
    * widgets this is basically position in widget Z-order.
    *  
    * @param   widget
    *          Widget to mover.
    * @param   pos
    *          Position in Z-order.
    */
   public void moveToZpos(Widget widget, int pos);
   
   /**
    * Select next focusable widget. 
    */
   public void nextFocus();

   /**
    * Select previous focusable widget.
    */
   public void prevFocus();

   /**
    * Remove specified widget from the widget list.
    * 
    * @param   widget
    *          Widget to remove.
    */
   public void remove(Widget widget);

   /**
    * Replace specified widget with another with preserving widget Z-order.
    * 
    * @param   oldWidget
    *          Old widget.
    * @param   newWidget
    *          New widget.
    */
   public void replace(Widget oldWidget, Widget newWidget);

   /**
    * Set focus to specified widget.
    * 
    * @param   focus
    *          Widget to set focus on.
    */
   public void setFocus(Widget focus);

   /**
    * Set container height.
    * 
    * @param   height
    *          New container height.
    */
   public void setHeight(int height);

   /**
    * Set container insets.
    * 
    * @param   insets
    *          Insets to set.
    */
   public void setInsets(Insets insets);

   /**
    * Set layout manager for the container.
    * 
    * @param   manager
    *          New layout manager.
    */
   public void setLayout(LayoutManager manager);

   /**
    * Get active layout manager for the container.
    * 
    * @return  Active layout manager.
    */
   LayoutManager getLayout();
   
   /**
    * Set container size.
    * 
    * @param   size
    *          Size to set.
    */
   public void setSize(Dimension size);

   /**
    * Set container size.
    * 
    * @param   width
    *          New width.
    * @param   height
    *          New height.
    */
   public void setSize(int width, int height);

   /**
    * Set container width.
    * 
    * @param   width
    *          Width to set.
    */
   public void setWidth(int width);

   /**
    * Indicates the beginning of the resize process. All repainting between
    * <code>startSizeChange</code> and <code>endSizeChange</code> calls is
    * suppressed.
    * 
    * @param    saveOldRect
    *           If <code>true</code> then the current bounding rectangle is
    *           saved for future repainting.
    */
   public void startSizeChange(boolean saveOldRect);

   /**
    * Get all widgets stored in the container.
    * 
    * @return  array of stored widgets.
    */
   public Widget[] widgets();

   /**
    * Set focus to first focusable widget.
    */
   public void firstFocus();

   /**
    * Set focus to last focusable widget.
    */
   void lastFocus();
}